/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
package uk.ac.ic.doc.kevlar.api.programs.framework.internal;


/**
 * A packet to represent control data ( at the moment you can only close a pipe).
 * @author tora
 */
final class AControlUnit implements IPacket {

    private static final long serialVersionUID = -7094517288336477388L;

    public static final int CLOSE_INPUT_PIPE = 0;
    public static final int CLOSE_OUTPUT_PIPE = 1;
    public static final int RECIEPT = 2;
    
    private final int control;
    private final String pipeName;
    private final long rptNum;
    
    public AControlUnit(String pipeName, int control) {
        this.control = control;
        this.pipeName = pipeName;
        this.rptNum = -1;
    }

    public AControlUnit(int control, long rptNum ) {
        this.rptNum = rptNum;
        this.control = control;
        this.pipeName = null;
    }
    
    /**
     * @see uk.ac.ic.doc.kevlar.api.programs.framework.internal.IPacket#getPipeName()
     */
    public String getPipeName() {
        return pipeName;
    }

    /**
     * @return The Control-Code for this packet.
     */
    public int getControlValue() {
        return control;
    }
    
    public long getRecieptNumber() {
        return rptNum;
    }

}
