/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 03-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.returnValues;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import uk.ac.ic.doc.kevlar.publicUtil.collections.IImmutableMap;
import uk.ac.ic.doc.kevlar.publicUtil.collections.IImmutableSet;
import uk.ac.ic.doc.kevlar.publicUtil.collections.ImmutableHashMap;
import uk.ac.ic.doc.kevlar.publicUtil.collections.ImmutableHashSet;
import uk.ac.ic.doc.kevlar.publicUtil.dom.XMLUtil;

/**
 * @author Daniel Burke
 *
 */
public final class ReturnModel implements IReturnModel {
	IImmutableSet<ReturnMapping> mappings;
	IImmutableMap<Integer, IReturnMapping> mappingsMap;
	
	public ReturnModel(Collection<IReturnMapping> mappings)
	{
		Set<ReturnMapping> tempMappings = new HashSet<ReturnMapping>();
		Map<Integer, IReturnMapping> tempMappingsMap = new HashMap<Integer, IReturnMapping>();
		for(IReturnMapping mapping : mappings)
		{
			if(!(mapping instanceof ReturnMapping))
			{
				throw new IllegalArgumentException("");
			}
			
			tempMappings.add((ReturnMapping)mapping);
			tempMappingsMap.put(Integer.valueOf(mapping.getValue()), mapping);
		}
		
		this.mappings = new ImmutableHashSet<ReturnMapping>(tempMappings);
		this.mappingsMap = new ImmutableHashMap<Integer, IReturnMapping>(tempMappingsMap);
	}
	
	public IImmutableMap<Integer, IReturnMapping>  getReturnValueMap()
	{
		return mappingsMap;
	}
	
	public String toString()
	{
	    return toString(0);
	}
	
	public String toString(int depth)
	{
	    StringBuilder text = new StringBuilder();
	    depth = XMLUtil.putOpenElem(text, "return-model", depth);
	    
	    depth = XMLUtil.putOpenElem(text, "return-mappings", depth);
	    
	    for(ReturnMapping mapping : mappings)
	    {
	        text.append(mapping.toString(depth));
	    }
	    
	    depth = XMLUtil.putCloseElem(text, "return-mappings", depth);
	    
	    depth = XMLUtil.putCloseElem(text, "return-model", depth);
	    return text.toString(); 
	}
}
