/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 24-Oct-2004
 *
 * Created by Steve Zymler
 */
package uk.ac.ic.doc.kevlar.core.framework;

import java.util.Map;
import java.util.Set;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IBasicArgumentInfo;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IInputOutputNodeTuple;
import uk.ac.ic.doc.kevlar.core.framework.typecheck.IInterfaceClass;
import uk.ac.ic.doc.kevlar.publicInterfaces.INodeID;
import uk.ac.ic.doc.kevlar.publicInterfaces.IProgramDescription;

/**
 * @author Daniel Burke and Steve Zymler
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public interface IPipelineProgram {
    
    /**
	 * @return a long that identifies the context holding a pipeline
	 * 			in the framework.
	 */
	long getID();	
	
	boolean isDestroyed();
    
    IArgumentInstance getArgumentInstance();
    
    IInputOutputNodeTuple getIONodes();
    
    /**
     * @return a non live copy of the arguments that are either available for the user to add, or the user has already added to the program
     */
	Set<String> getAvailableArguments();
	
	Set<String> getInvalidArguments();
	
	IProgramDescription getDescription();
	
	IBasicArgumentInfo getArgumentInfo(String argName);

	Map<INodeID, IPipe> getConnectedInputPipes();
	
	Map<INodeID, IPipe> getConnectedOutputPipes();
	
	Map<String, IInterfaceClass> getLowerBounds();
	
	String getCanonicalName();
	
	boolean inputHasConnectedPipe(INodeID id);
	
	boolean outputHasConnectedPipe(INodeID id);
	
	IPipe getInputPipe(INodeID id);
	
	IPipe getOutputPipe(INodeID id);
	
	IInterfaceClass getOutputNodeType(INodeID outputNodeID );
	
	void addExecutionEndedListener(IExecutionEndedListener listener);
	
	void kill();
}
