/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 20-Oct-2004
 *
 */
package uk.ac.ic.doc.kevlar.core.framework;

import uk.ac.ic.doc.kevlar.core.framework.typecheck.IInterfaceClass;
import uk.ac.ic.doc.kevlar.publicInterfaces.IType;

/**
 * @author Daniel Burke and Steve Zymler
 *
 */
public final class InterfaceClass implements IInterfaceClass 
{
	private Class<? extends IType> type ;
	
	public InterfaceClass( Class<? extends IType> type )
	{
		if( !IType.class.isAssignableFrom(type) )
		{
			throw new IllegalArgumentException("type parameter does not extend IType");
		}
		
		if(type == null)
		{
			throw new IllegalArgumentException("type was null");
		}
		
		if(!type.isInterface())
		{
			throw new IllegalArgumentException("type must represent an interface");
		}
		
		this.type = type ;
	}

	public Class<? extends IType> getTypeClass() 
	{		
		return type;
	}
	
	public boolean equals( Object object )
	{
		if( object instanceof InterfaceClass )
		{
			return	type.equals( 
					((InterfaceClass) object).getTypeClass() );			
		}
		return false ;
	}
		
	public int hashCode() 
	{
		return type.hashCode();
	}	
	
	public String toString()
	{
		return type.getName() ;
	}
}
