/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
package uk.ac.ic.doc.kevlar.core.framework.internal;

import uk.ac.ic.doc.kevlar.core.framework.internal.programs.IProgramExitListener;


/**
 * Represents a handle to a running program
 * @author tora
 */
public interface IProgramInstance {

    /**
     * @return true iff this IProgramInstance is running at the moment.
     */
    boolean isRunning();

    /**
     * Kill this program outright. Returns once the program is dead.
     *
     */
    void kill();
    
    /**
     * Blocks till this process exits ( unless it has already exited ) and gives back its return value.
     * @return a representation of the return value of the program.
     */
    IReturnValue getReturnValue();
    
    /**
     * Adds a notifier that runs when the program instance exits.
     * If the program has exited already, the notifier is called back
     * immediately, otherwise it is called back upon the program exiting.
     * @param notifer The notifier to be notified when this IProgramInstance exits.
     */
    void addExitNotifier( IProgramExitListener notifer );
}
