/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on Dec 21, 2004
 */
package uk.ac.ic.doc.kevlar.core.programs.coreUtils;

import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.publicInterfaces.AbstractProgram;
import uk.ac.ic.doc.kevlar.publicInterfaces.ICloseEventListener;
import uk.ac.ic.doc.kevlar.publicInterfaces.IInputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IOutputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IPipeEnd;

/**
 * @author tora
 */
public class TimeOutRepeater extends AbstractProgram {

    final AtomicInteger timeLeft = new AtomicInteger();
    
    int timeout;
    
    final Runnable run = new Runnable () {
        
        public void run() {
            
            while ( timeLeft.getAndDecrement() > 0 ) {
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                    /* */
                }
            }
            
            exit(0);
        }
        
        
    };
    
    public void run(IArgumentInstance args, Map<String, IInputPipe> inputs,
            Map<String, IOutputPipe> outputs) {
        String time = args.getText("TimeOut");

        timeout = getTime(time,3000);

        timeLeft.set(timeout);
        
        Thread t = new Thread( run );
        
        //t.setPriority(Thread.NORM_PRIORITY-1);
        t.setDaemon(true);
        
        IInputPipe iip = inputs.get("In");
        IOutputPipe iop = outputs.get("Out");

        if (iip == null || iop == null) {
            exit(0);
            return;
        }
        
        ICloseEventListener exitOnClose = new ICloseEventListener() {
            public void pipeClosed(IPipeEnd source) {
                exit(0);
            }
        };        
        
        iip.addCloseEventListener(exitOnClose);
        iop.addCloseEventListener(exitOnClose);

        t.start();
        
        try {
            while (true) {
                iop.write(iip.read());
                timeLeft.set(timeout);
            }
        } catch (InterruptedException e) {
            exit(0);
        }

        
    }

    
    private int getTime( String time, int def ) {
        if( time == null ) {
            return def;
        }
        
        try { 
            return Integer.parseInt(time);
        } catch ( NumberFormatException nfe ) {
            return def;
        }
    }
}
