/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 26-Nov-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.programs.textUtils;

import java.util.Map;
import java.util.regex.PatternSyntaxException;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.core.types.textType.ITextType;
import uk.ac.ic.doc.kevlar.publicInterfaces.AbstractProgram;
import uk.ac.ic.doc.kevlar.publicInterfaces.ICloseEventListener;
import uk.ac.ic.doc.kevlar.publicInterfaces.IInputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IOutputPipe;
import uk.ac.ic.doc.kevlar.publicInterfaces.IPipeEnd;

/**
 * @author toa02
 * 
 */
public class TextSearch extends AbstractProgram {

    /**
     * @see uk.ac.ic.doc.kevlar.publicInterfaces.AbstractProgram#run(uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance,
     *      java.util.Map, java.util.Map)
     */
    public void run(IArgumentInstance arg0, Map<String, IInputPipe> arg1,
            Map<String, IOutputPipe> arg2) {

        String pattern = arg0.getText("Pattern");

        if (pattern == null) {
            pattern = "";
        }

        IInputPipe iip = arg1.get("In");
        IOutputPipe iop = arg2.get("Out");

        if (iip == null || iop == null) {
            exit(0);
            return;
        }

        ICloseEventListener exitOnClose = new ICloseEventListener() {

            public void pipeClosed(IPipeEnd source) {
                exit(0);
            }
        };

        iip.addCloseEventListener(exitOnClose);
        iop.addCloseEventListener(exitOnClose);

        try {
            while (true) {
                ITextType type = (ITextType) iip.read();
                if (type.getText().matches(pattern)) {
                    iop.write(type);
                }
            }
        } catch (InterruptedException e) {
            exit(1);
        } catch (PatternSyntaxException e ) {
            exit(2);
        }
         

    }

}
