/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 23-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.keyboard;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.ProgramAutoCompleteState;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.ISelectListener;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.listbox.ListBox;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.listbox.ListEntry;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox.IModifyListener;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox.ITextBox;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox.TextBox;

/**
 * @author Marc
 **/
public class AutoCompleteBox extends AMouseListenerObject implements KeyListener, IModifyListener {

//    protected SWTTextBox textbox;
    protected TextBox textbox;
    protected ListBox listbox;
    protected IAutoCompleteListener listener;
    
    public AutoCompleteBox() {
        
        super();
        
        createTextBox();
        listbox = null;
        
    }
    
    public void createTextBox() {
        
/*        textbox = new SWTTextBox(SWT.SINGLE);
        textbox.setPosition(new Point(5, 2));
        textbox.setSizeFillHorizontal(new Point(2, 13));
        textbox.getTextBox().addKeyListener(this);
        textbox.getTextBox().addModifyListener(this);
        addChild(textbox); */
        
        textbox = new TextBox();
        textbox.setPosition(new Point(0, 0));
        textbox.setSizeFillHorizontal(new Point(0, 20));
        textbox.setModifyListener(this);
/*        textbox.getTextBox().addKeyListener(this);
        textbox.getTextBox().addModifyListener(this); */
        addChild(textbox);
        
    }
    
    public TextBox getTextBox() {
        
        return textbox;
        
    }
    
    public void moveSelectionUp() {
        
        listbox.moveSelectionUp();
        
    }
    
    public void moveSelectionDown() {

        listbox.moveSelectionDown();
        
    }
    
    public void setListBox(ListBox l) {
        
        if (listbox != null) {
            
            removeChild(listbox);
            
        }
        
        listbox = l;
        listbox.setPosition(new Point(0, 20));
        listbox.setSizeFillBoth(new Point(0, 0));
        addChild(listbox);
        
    }
    
    public void show() {
        
        super.show();
        
/*        if (textbox == null) {
            
            createTextBox();
            
        } */
        
    }
    
    public void hide() {
        
        super.hide();
        
/*        if (textbox != null) {
        
            textbox.dispose();
            textbox = null;
            
        } */
        
    }
    
    public void addText(String text) {
        
/*        if (textbox != null) {
        
            Text textwidget = textbox.getTextBox(); 
            textwidget.setText(textwidget.getText()+text);
            textwidget.setSelection(text.length());
            listbox.setFilter(text);
            
        } */
        
        textbox.setText(textbox.getText()+text);
        textbox.end();
//        textwidget.setSelection(text.length());
//        listbox.setFilter(text);
                
    }
    
    public void setListener(IAutoCompleteListener l) {
        
        listener = l;
        
    }
    
    public Object getSelected() {
        
        ListEntry selectedEntry = listbox.getEntry(listbox.getSelectedIndex());
        
        if (selectedEntry != null) {
        
            return selectedEntry.getData();
            
        }
        
        return null;
        
    }

    public void keyPressed(KeyEvent arg0) {

/*        if (arg0.keyCode == 32 && listener != null && listbox != null) {
            
            arg0.doit = false;
            ListEntry selectedEntry = listbox.getEntry(listbox.getSelectedIndex());
            
            if (selectedEntry != null) {
            
                listener.entryCompleted(selectedEntry.getData());
                
            }
            
        } else if (arg0.keyCode == SWT.ESC && listener != null) {
            
            arg0.doit = false;
            listener.entryCancelled();
         
        } else */
        
        if (listbox != null && textbox != null && arg0.keyCode == SWT.ARROW_DOWN) {
            
            arg0.doit = false;
            int selected = listbox.getSelectedIndex();
            selected++;            
            
            if (selected == listbox.entryCount()) {
                
                selected = 0;
                
            }
            
            listbox.selectEntry(selected);

        } else if (listbox != null && textbox != null && arg0.keyCode == SWT.ARROW_UP) {
            
            arg0.doit = false;
            int selected = listbox.getSelectedIndex();
            
            if (selected == 0) {
                
                selected = listbox.entryCount();
                
            }
            
            selected--;            
            listbox.selectEntry(selected);
            
        }
        
    }

    public void keyReleased(KeyEvent arg0) {
        
        
    }
    
    public void drawBefore(GC gc, Rectangle redrawArea) {
        
//        Utils.drawBox(gc, new Rectangle(-redrawArea.x, -redrawArea.y, area.width, 20), EDIT_BOX_SKIN);
        
    }

/*    public void modifyText(ModifyEvent arg0) {
        
        if (textbox != null) {
        
//            listbox.setFilter(textbox.getTextBox().getText());
            listbox.setFilter(textbox.getText());
            
        }
        
    } */

    public void onModify(ITextBox textbox, String text) {

        listbox.setFilter(text);      
        listbox.selectEntry(0);
        
    }

}
