/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 27-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.keyboard.actions.autocomplete.programentry;

import org.eclipse.swt.events.KeyEvent;

import uk.ac.ic.doc.kevlar.api.programs.IProgramInfoNode;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IBasicArgumentInfo;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.Item;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyBindings;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.actions.IAction;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.ProgramEntryAutoCompleteState;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.State;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.AIONodeEntry;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.AProgramEntry;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.PipeLineManager;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.Program;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.arguments.Argument;
import uk.ac.ic.doc.kevlar.gui.internal.popup.PopUpBox;
import uk.ac.ic.doc.kevlar.gui.internal.popup.PopUpManager;

/**
 * @author Marc
 **/
public class SelectProgramEntryAction implements IAction {

    public boolean perform(Item event, State s, KeyEvent e) {
        
        if (s instanceof ProgramEntryAutoCompleteState) {
            
            ProgramEntryAutoCompleteState state = (ProgramEntryAutoCompleteState)s;
            Object entry = state.getAutoComplete().getSelected();
            PopUpBox popup = state.getPopUp();
            
            KeyboardManager.getSingleton().setState(state.getHistory());
            
            if (popup != null) {
            
                if (entry instanceof IProgramInfoNode) {
                    
                    IProgramInfoNode infoNode = (IProgramInfoNode)entry;
                    Program program = state.getProgram();
                    
                    if (entry instanceof IBasicArgumentInfo) {
                        
                        program.addArgument((IBasicArgumentInfo)infoNode);
                        
                    }
                    
                    AProgramEntry programEntry = program.getEntry(infoNode.getName());
                    
                    if (programEntry != null) {
                        
                        if (programEntry instanceof Argument) {
                            
                            ((Argument)programEntry).showEditor();
                            
                        } else if (programEntry instanceof AIONodeEntry) {
                            
                            PipeLineManager.getSingleton().selectNode(((AIONodeEntry)programEntry));
                            
                            if (new Item(e.keyCode).equals(KeyBindings.code("ENTER"))) {
                                
                                PipeLineManager.getSingleton().deselectProgram();
                                
                            }
                            
                        }
                        
                    }
                    
                } else if (entry instanceof AIONodeEntry) {
                    
                    PipeLineManager.getSingleton().selectNode(((AIONodeEntry)entry));
                    
                    if (new Item(e.keyCode).equals(KeyBindings.code("ENTER"))) {
                        
                        PipeLineManager.getSingleton().deselectProgram();
                        
                    }
                    
                }

                PopUpManager.getSingleton().hidePopUp(popup);
                
            }
            
        }
        
        return false;
        
    }

}
