/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 14-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.popup;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.CanvasManager;

/**
 * @author Marc
 **/
public class PopUpManager extends AMouseListenerObject {

    private static PopUpManager singleton;
    
    private ADrawableObject popup;
    private IPopUpHideListener hideListener;
    
    public PopUpManager() {
        
        super();
        popup = null;
        
    }
    
    public static void createSingleton() {
        
        if (singleton == null) {
            
            singleton = new PopUpManager();
            
        }
        
    }
    
    public static PopUpManager getSingleton() {
        
        return singleton;
        
    }
    
    public ADrawableObject getCurrentPopUp()
    {
    	return popup;
    }
    
    public void mousePress(Point position, int button) {
        
        if (popup != null && !popup.getArea().contains(position)) {
            
            hidePopUp(popup);
            
        }
        
    }
    
    public void hidePopUp(ADrawableObject p) {
    	
    	if (popup == p) {

    	    hidePopUp();
            
    	}
    	
    }
    
    public void hidePopUp() {
        
    	if (popup != null) {
    		
//            popup.hide();
    	    popup.redraw();
            removeChild(popup);
            ADrawableObject temp = popup;
            popup = null;
            
            if (hideListener != null) {
                
                hideListener.popUpHidden(temp);
                hideListener = null;
                
            }
    		            
            releaseMouseFocus();
            
    	}
    	        
    }
    
    public void showPopUp(ADrawableObject p, IPopUpHideListener l) {
        
        showPopUp(p);
        hideListener = l;
        
    }
    
    public void showPopUp(ADrawableObject p) {
        
        if (popup != null) {
            
/*            if (popup.isVisible()) {
            
                popup.hide();
                
            }
            
            removeChild(popup); */
            
            hidePopUp();
            
        }
        
        popup = p;
        Rectangle popupArea = popup.getArea();
        Rectangle canvasArea = CanvasManager.getSingleton().getArea();
        Point newPosition = new Point(popupArea.x, popupArea.y);
        
        if (popupArea.x < 0) {
            
            newPosition.x = 0;
            
        } else if (popupArea.x + popupArea.width > canvasArea.width) {
            
            newPosition.x = popupArea.x-popupArea.width+1;
            
        }
        
        if (popupArea.y < 0) {
            
            newPosition.y = 0;
            
        } else if (popupArea.y + popupArea.height > canvasArea.height) {
            
            newPosition.y = popupArea.y-popupArea.height+1;
            
        }
        
        setPosition(newPosition);
        setSize(popup.getSize());
        popup.setPosition(new Point(0, 0));
        addChild(popup);
//        popup.show();
        
        getMouseFocus();
        
    }
    
    public void childDimensionsChanged(ADrawableObject child) {
        
        if (child == popup && (popup.getSize().x != getSize().x || popup.getSize().y != getSize().y)) {
            
            setSize(popup.getSize());
            
        }
        
    }
        
}
