/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 01-Jan-2005
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets;

import java.io.FileNotFoundException;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.caches.ImageCache;

/**
 * @author Daniel Burke
 *
 */
public class RollOverButtonWithDisable extends RollOverButton
{
    private Image disabledImage;
    boolean disabled;
    
    public RollOverButtonWithDisable(String image, String rolloverImage, String downImage, String disabledImage)
    {
        super(image, rolloverImage, downImage);
        setDisabledImage(disabledImage);
    }
    
    public void setDisabledImage(String disabledImage)
    {
        try {
            this.disabledImage = ImageCache.get(disabledImage);
        } catch (FileNotFoundException e) {
            this.disabledImage = null;
        }
    }
    
    public void setDisabledImage(Image disabledImage)
    {
        this.disabledImage = disabledImage;
    }

    public void setDisabled(boolean disabled)
    {
        this.disabled = disabled;
        redraw();
    }
    
    public boolean getDisabled()
    {
        return disabled;
    }
    
    public void mouseRelease(Point position, int mouseButton)
    {
        if(disabled)
        {
            return;
        }
        super.mouseRelease(position, mouseButton);
    }
    
    public void drawBefore(GC gc, Rectangle redrawArea)
    {
        if(disabled)
        {
            if(disabledImage != null)
            {
                gc.drawImage(disabledImage, -redrawArea.x, -redrawArea.y);
            }
            return;
        }
        
        super.drawBefore(gc, redrawArea);
    }
}
