/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 28-Oct-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;


/**
 * @author Marc
 **/
public class StretchyBox extends AMouseListenerObject {

    protected boolean stretchX;
    protected boolean stretchY;
    protected Point minSize;
    protected Point padding;
    protected boolean update;
    
    public StretchyBox(boolean stretchX, boolean stretchY) {
        
        super();
        this.stretchX = stretchX;
        this.stretchY = stretchY;
        this.minSize = new Point(0, 0);
        this.padding = new Point(0, 0);
        this.update = true;
        
    }
    
    public void noUpdates() {
        
        update = false;
        
    }
    
    public void update() {
        
        update = true;
        updateDimensions();
        
    }
    
    public void setMinimumSize(Point minSize) {
        
        this.minSize = minSize;
        updateDimensions();
        
    }
    
    public void setPadding(Point padding) {
        
        this.padding = padding;
        updateDimensions();
        
    }
    
    public void updateDimensions() {
     
        if (update) {
        
            redraw();
            
            Point oldSize = new Point(area.width, area.height);
            
            area.x = alignedArea.x;
            area.y = alignedArea.y;
            
            if (stretchX) {
                
                area.width = minSize.x;
                
            } else {
                
                area.width = alignedArea.width;
                
            }
            
            if (stretchY) {
                
                area.height = minSize.y;
                
            } else {
                
                area.height = alignedArea.height;
                
            }
            
            for (int i=0; i<children.size(); i++) {
                
                Rectangle childArea = children.get(i).getArea();
                
                if (stretchX) {
                    
                    area.width = Math.max(area.width, childArea.x+childArea.width);
                    
                }
                
                if (stretchY) {
                    
                    area.height = Math.max(area.height, childArea.y+childArea.height);
                    
                }
                
            }
            
            area.width = area.width + padding.x;
            area.height = area.height + padding.y;
            
            if (oldSize.x != area.width || oldSize.y != area.height) {
                
                for (int i=0; i<children.size(); i++) {
                    
                    ADrawableObject child = children.get(i);
                    
                    if (stretchX && child.isAlignedRight()) {
                        
                        System.err.println("WARNING: Child of horizontally stretching StretchyBox is aligned to the right!");
                        
                    } else if (stretchX && child.isFillHorizontal()) {
                        
                        System.err.println("WARNING: Child of horizontally stretching StretchyBox is set to fill horizontal!");
                        
                    } else if (stretchY && child.isAlignedBottom()) {
                        
                        System.err.println("WARNING: Child of vertically stretching StretchyBox is aligned to the bottom!");
                        
                    } else if (stretchY && child.isFillVertical()) {
                        
                        System.err.println("WARNING: Child of vertically stretching StretchyBox is set to fill vertical!");
                        
                    } else {
                        
                        child.parentResized(new Point(area.width, area.height));
                        
                    }
                    
                }
                
                if (parent != null) {
                    
                    parent.childDimensionsChanged(this);
                    
                }
                
            }
            
            redraw();
            
        }
        
    }
    
    public void addChild(ADrawableObject child) {
    	
    	super.addChild(child);
    	updateDimensions();
    	
    }
    
    public void childDimensionsChanged(ADrawableObject child) {
	    
	    updateDimensions();
	    
	}
    
}
