/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 29-Oct-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars;

import org.eclipse.swt.graphics.Point;

import uk.ac.ic.doc.kevlar.gui.internal.widgets.IScrollingWindow;


/**
 * @author Marc
 **/
public class HorizontalScrollBar extends AScrollbar {

    public HorizontalScrollBar(IScrollingWindow window) {
        
//        super(new Point(0, SCROLLBAR_HEIGHT), false, true);
        super(window);
        
        setPosition(new Point(0, AScrollbar.SCROLLBAR_SIZE));
        setAlignment(false, true);
        setSizeFillHorizontal(new Point(0, AScrollbar.SCROLLBAR_SIZE));
        
//        this.window = window;
        
        button1 = new ScrollLeftButton(this);
        addChild(button1);
        
        button2 = new ScrollRightButton(this);
        addChild(button2);
        
        slider = new HorizontalScrollBarSlider(this);
        addChild(slider);
        
        value = 0;
        size = 1;
        max = 5;
        increment = 1;
        
    }
    
    
    public void unsetGap()
    {
        if(alignBottom)
        {
            setPosition(new Point(0, AScrollbar.SCROLLBAR_SIZE));
        }
        else
        {
            setPosition(new Point(0, 0));
        }
        setSizeFillHorizontal(new Point(0, AScrollbar.SCROLLBAR_SIZE));
    }
    

    public void setGapLeft() {

        if (alignBottom) 
        {
            setPosition(new Point(AScrollbar.SCROLLBAR_SIZE, AScrollbar.SCROLLBAR_SIZE));
        } 
        else 
        { 
            setPosition(new Point(AScrollbar.SCROLLBAR_SIZE, 0));          
        }
        setSizeFillHorizontal(new Point(0, AScrollbar.SCROLLBAR_SIZE)); 
    }
    
    public void setGapRight() {
        if (alignBottom) 
        {
            setPosition(new Point(0, AScrollbar.SCROLLBAR_SIZE));
        } 
        else 
        { 
            setPosition(new Point(0, 0));          
        }
        setSizeFillHorizontal(new Point(AScrollbar.SCROLLBAR_SIZE, AScrollbar.SCROLLBAR_SIZE));
        
    }
    
    public void setAlignmentTop() {
        
        setPosition(new Point(0, 0));
        setAlignment(true, true);
        
    }
    
    public void updateWindow() {
        
        redraw();
        window.setHorizontalPosition(value);
        
    }
    
    public void updateFromWindow() {
        
        value = window.getHorizontalPosition();
        updateSlider(); 
        
    }
    
    public int getSpace() {
        
        return area.width-AScrollbar.SCROLLBAR_SIZE*2;
        
    }
     
    public void mousePress(Point position, int button) {
        
/*        int availableSpace = getSpace();
        double barSize = 0;
                
        if (size >= max) {
            
            barSize = availableSpace;
            
        } else {
        
            barSize = size*(double)availableSpace/max;
            
        }
        
        setValue((int)((position.x-button1.getSize().x)*(max-size)/(availableSpace-barSize)-barSize/2)); */
        
        setValue(realValueToScrollbarValue(position.x-button1.getSize().x)-size/2);
                
    }
    
    public boolean capturesMouse() {
        
        return true;
        
    }
                
}
