/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 28-Oct-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars;

import org.eclipse.swt.graphics.Point;

import uk.ac.ic.doc.kevlar.gui.internal.widgets.IScrollingWindow;


/**
 * @author Marc
 **/
public class VerticalScrollBar extends AScrollbar {

    public VerticalScrollBar(IScrollingWindow window) {
        
        super(window);
        
        setPosition(new Point(AScrollbar.SCROLLBAR_SIZE, 0));
        setAlignment(true, false);
        setSizeFillVertical(new Point(AScrollbar.SCROLLBAR_SIZE, 0));

//        this.window = window;
        
        button1 = new ScrollUpButton(this);
        addChild(button1);
        
        button2 = new ScrollDownButton(this);
        addChild(button2);
        
        slider = new VerticalScrollBarSlider(this);
        addChild(slider);
        
        value = 0;
        size = 1;
        max = 5;
        increment = 1;
        
    }
    
    public void unsetGap()
    {
        if(alignRight)
        {
            setPosition( new Point(AScrollbar.SCROLLBAR_SIZE, 0));
        }
        else
        {
            setPosition( new Point(0, 0));
        }
        
    	setSizeFillVertical(new Point(AScrollbar.SCROLLBAR_SIZE, 0));
    }
    
    public void setGapTop() {

        if(alignRight)
        {
            setPosition(new Point(AScrollbar.SCROLLBAR_SIZE, AScrollbar.SCROLLBAR_SIZE));
        }
        else
        {
            setPosition(new Point(0, AScrollbar.SCROLLBAR_SIZE));
        }
        
        setSizeFillVertical(new Point(AScrollbar.SCROLLBAR_SIZE, 0));
        
    }
    
    public void setGapBottom() {

        if(alignRight)
        {
            setPosition(new Point(AScrollbar.SCROLLBAR_SIZE, 0));
        }
        else
        {
            setPosition(new Point(0, 0));
        }
        
        setSizeFillVertical(new Point(AScrollbar.SCROLLBAR_SIZE, AScrollbar.SCROLLBAR_SIZE));
        
    }
    
    public void setAlignmentLeft() {
        
        setPosition(new Point(0, 0));
        setAlignment(false, false);
        
    }

    public void updateWindow() {
        
        redraw();
        window.setVerticalPosition(value);
        
    }
    
    public void updateFromWindow() {
        
        value = window.getVerticalPosition();
        updateSlider(); 
        
    }
        
    public int getSpace() {
        
        return area.height-AScrollbar.SCROLLBAR_SIZE*2;
        
    }
    
    public void mousePress(Point position, int button) {
    
        setValue(realValueToScrollbarValue(position.y-button1.getSize().y)-size/2);
        
    }
    
    public boolean capturesMouse() {
        
        return true;
        
    }
        
}
