/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
package uk.ac.ic.doc.kevlar.publicInterfaces;

/**
 * @author tora
 * Contract: Implemenations must override equals so that
 * it is equal to INodeID if getFriendlyName() is the same,
 * and INode if getName() is equal to getFriendlyName()
 * Therefore hashcode should be the name's hashcode
 */
public interface INodeID {

    /*
     * toa02: I need a way of identifying things across a stream: either the
     * getFriendlyName() should be unique, or this interface should be
     * serializable. Any comments?
     */

    /**
     * Compares this NodeID with another and returns true iff they are the same.
     * 
     * @param other
     *            Another nodeID to compare this one with.
     * @return true iff this node equals another nodeID.
     */
    public boolean equals(INodeID other);

    /**
     * @see java.lang.Object#hashCode() Must be overridden so equals also works.
     * @return the int hashcode of this object.
     */
    public int hashCode();

    /**
     * @return the friendly name of the Node. These should be unique to an
     *         individual program.
     */
    public String getFriendlyName();
}
